/* Yet Another GDA Config Tool
* Copyright (C) 2002 Aditel (http://www.aditel.org)
*
* AUTHORS:
*	Emilio A. Sanchez Lopez <esanchez@aditel.org>
*	Javier Muoz Ferrara <javierm@aditel.org>
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License as
* published by the Free Software Foundation; either version 2 of the
* License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Library General Public License for more details.
*
* You should have received a copy of the GNU General Public
* License along with this program; see the file COPYING.  If not,
* write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
* Boston, MA 02111-1307, USA.
*/

#include <popt.h>
#include <glib.h>
#include <libgda/libgda.h>

typedef struct
{
	gboolean list_providers;
	gboolean list_datasources;
	gboolean test_all;
	gchar *remove;
	gchar *test;
	gchar *username;
	gchar *password;
	gchar *description;
	gchar *cns;
	gchar *provider;
	gchar *dsn;
} CmdArguments;

CmdArguments cmdArgs;

static void print_provider(const GdaProviderInfo *info)
{
	GList *l;

	g_print("Id: %s\n",info->id);
	g_print("Location: %s\n",info->location);
	g_print("Description: %s\n",info->description);
	g_print("Accepts the following arguments in the GDA data source's DSN string:\n");
	for(l=info->gda_params;l!=NULL;l=l->next)
	{
		GdaParameter *p=(GdaParameter *) l->data;
		g_print("%s ",p);
	}
	g_print("\n");
}

static void list_providers()
{
	GList *provider_list;
	GList *l;

	provider_list=gda_config_get_provider_list();

	g_print("Providers:\n");
	g_print("-------------------------------\n");
	for(l=provider_list;l!=NULL;l=l->next)
	{
		GdaProviderInfo *info=(GdaProviderInfo *) l->data;
		print_provider(info);
		g_print("-------------------------------\n");
	}
	g_print("End of providers.\n");

	gda_config_free_provider_list(provider_list);
}

static void print_datasource(const GdaDataSourceInfo *info)
{
	g_print("Datasource name: %s\n",info->name);
	g_print("Provider: %s\n",info->provider);
	g_print("Connection String: %s\n",info->cnc_string);
	g_print("Description: %s\n",info->description);
	g_print("User name: %s\n",info->username);
	g_print("Password: %s\n",info->password);
}

static void list_datasources()
{
	GList *datasource_list;
	GList *l;

	datasource_list=gda_config_get_data_source_list();

	g_print("Datasources:\n");
	g_print("-------------------------------\n");
	for(l=datasource_list;l!=NULL;l=l->next)
	{
		GdaDataSourceInfo *info=(GdaDataSourceInfo *) l->data;
		print_datasource(info);
		g_print("-------------------------------\n");
	}
	g_print("End of datasources.\n");

	gda_config_free_data_source_list(datasource_list);
}

static void remove_datasource()
{
	GdaDataSourceInfo *dinfo;

	dinfo=gda_config_find_data_source(cmdArgs.remove);

	if (dinfo==NULL)
	{
		g_printerr("Error: Datasource %s doesn't exist\n",cmdArgs.remove);
		exit(-1);
	}
	gda_config_free_data_source_info(dinfo);
	
	gda_config_remove_data_source(cmdArgs.remove);

	dinfo=gda_config_find_data_source(cmdArgs.remove);

	if (dinfo==NULL)
	{
		g_print("Data source %s removed successfully\n",cmdArgs.remove);
	}
	else
	{
		g_printerr("Data source %s was not removed\n",cmdArgs.remove);
		exit(-1);
	}
}

static void config_datasource()
{
	GdaProviderInfo *pinfo;
	GdaDataSourceInfo *dinfo;

	pinfo=gda_config_get_provider_by_name(cmdArgs.provider);

	if (pinfo==NULL)
	{
		g_printerr("Error: Couldn't locate provider: %s\n",cmdArgs.provider);
		exit(-1);
	}

	gda_config_free_provider_info(pinfo);

	dinfo=gda_config_find_data_source(cmdArgs.dsn);

	if (dinfo==NULL)
	{
		g_print("Adding a new datasource...\n");
	}
	else
	{
		g_print("\nOld datasource:\n");
		print_datasource(dinfo);
		g_print("\nUpdating datasource...\n\n");
		gda_config_free_data_source_info(dinfo);
	}


	gda_config_save_data_source(cmdArgs.dsn,cmdArgs.provider,
			cmdArgs.cns,cmdArgs.description,
			cmdArgs.username,cmdArgs.password);

	dinfo=gda_config_find_data_source(cmdArgs.dsn);
	if (dinfo!=NULL)
	{
		print_datasource(dinfo);
		g_print("Datasource %s saved.\n",dinfo->name);
		gda_config_free_data_source_info(dinfo);
	 }
	else
	{
		g_printerr("Couldn't save datasource %s",cmdArgs.dsn);
		exit(-1);
	}
}


static void test_datasource(const gchar *dsn)
{
	GdaDataSourceInfo *dinfo;
	GdaClient *client;
	GdaConnection *cnc;

	dinfo=gda_config_find_data_source(dsn);

	g_print("Datasource %s: ",dsn);
	if (dinfo==NULL)
	{
		g_print("doesn't exist in configuration.\n");
	}
	else
	{
		g_print("exists in configuration ");

		client = gda_client_new();
		if (!GDA_IS_CLIENT(client))
		{
			g_printerr("Error: Couldn't create libgda client\n");
			exit(-1);
		}

		cnc = gda_client_open_connection(client, dinfo->name, dinfo->username, dinfo->password);
		if (!GDA_IS_CONNECTION (cnc))
		{
			g_print("but the connection attempt was failed\n");
		}
		else
		{
			g_print("and the connection attempt was successful\n");
			gda_connection_close(cnc);
		}

		gda_config_free_data_source_info(dinfo);
	}

}

static void test_all_datasources()
{
	GList *datasource_list;
	GList *l;

	datasource_list=gda_config_get_data_source_list();

	for(l=datasource_list;l!=NULL;l=l->next)
	{
		GdaDataSourceInfo *info=(GdaDataSourceInfo *) l->data;

		test_datasource(info->name);
	}

	gda_config_free_data_source_list(datasource_list);
}

static void run_gda_config(gpointer user_data)
{
	GdaClient *GdaObjetClient;

	GdaObjetClient=gda_client_new();
	if (!GDA_IS_CLIENT(GdaObjetClient))
	{
		g_printerr("Error: Couldn't create GdaClient\n");
		exit(-1);
	}

	if (cmdArgs.list_datasources)
	{
		list_datasources();
	}

	if (cmdArgs.list_providers)
	{
		list_providers();
	}

	if (cmdArgs.test) 
	{
		g_print("Testing %s datasource\n",cmdArgs.test);
		test_datasource(cmdArgs.test);
		g_print("Test done.\n");
	}
	else if (cmdArgs.test_all)
	{
		g_print("Testing all datasources\n");
		test_all_datasources();
		g_print("Test done.\n");
	}
	else if (cmdArgs.remove)
	{
		remove_datasource();
	}
	else if (cmdArgs.dsn)
	{
		config_datasource();
	}

	gda_main_quit();
}

options (int argc, const char **argv)
{
	poptContext context;
	int out=0;

	static const struct poptOption options[] = {
		{
			"list-datasources",
			'L',
			POPT_ARG_NONE,
			&cmdArgs.list_datasources,
			0,
			"Show avaliable datasources",
		},
		{
			"list-providers",
			'l',
			POPT_ARG_NONE,
			&cmdArgs.list_providers,
			0,
			"Show avaliable providers"
		},
		{
			"remove",
			'r',
			POPT_ARG_STRING,
			&cmdArgs.remove,
			0,
			"Remove datasource",
			"datasource"
		},
		{
			"test",
			't',
			POPT_ARG_STRING,
			&cmdArgs.test,
			0,
			"Test datasource",
			"datasource"
		},
		{
			"test-all",
			'T',
			POPT_ARG_NONE,
			&cmdArgs.test_all,
			0,
			"Test all configured datasources",
		},
		{
			"datasource",
			'D',
			POPT_ARG_STRING,
			&cmdArgs.dsn,
			0,
			"Datasource name",
			"datasource"
		},
		{
			"provider",
			'P',
			POPT_ARG_STRING,
			&cmdArgs.provider,
			0,
			"Provider name",
			"provider"	
		},
		{
			"cns",
			'c',
			POPT_ARG_STRING,
			&cmdArgs.cns,
			0,
			"Connection string",
			"connection_string"
		},
		{
			"description",
			'd',
			POPT_ARG_STRING,
			&cmdArgs.description,
			0,
			"Datasource description",
			"description"
		},
		{
			"user",
			'u',
			POPT_ARG_STRING,
			&cmdArgs.username,
			0,
			"User name",
			"user"
		},
		{
			"password",
			'p',
			POPT_ARG_STRING,
			&cmdArgs.password,
			0,
			"Password",
			"password"
		},
		POPT_AUTOHELP
		POPT_TABLEEND
	};

	context = poptGetContext (NULL, argc, argv, options, 0);
	if (!context)
	{
		g_printerr ("Error: Couldn't create the poptcontext\n");
		exit (-1);
	}

	if (poptGetNextOpt (context) < -1 || poptGetArg (context) != NULL || argc==1)
	{
		g_printerr ("Error: Incorrect argument in command line.\n");
		poptPrintHelp (context, stderr, 0);
		poptFreeContext (context);
		exit (-1);
	}

	if ((cmdArgs.remove) && (argc!=3))
	{
		g_printerr("Error: --remove option couldn't appear with other options\n");
		exit(-1);
	}

	if ((cmdArgs.test) && (argc!=3))
	{
		g_printerr("Error: --test option couldn't appear with other options\n");
		exit(-1);
	}

	if ((cmdArgs.test_all) && (argc!=2))
	{
		g_printerr("Error: --test-all option couldn't appear with other options\n");
		exit(-1);
	}

	if ((cmdArgs.list_providers) && (argc!=2))
	{
		g_printerr("Error: --list-providers option couldn't appear with other options\n");
		exit(-1);
	}

	if ((cmdArgs.list_datasources) && (argc!=2))
	{
		g_printerr("Error: --list-datasources option couldn't appear with other options\n");
		exit(-1);
	}

	if ((cmdArgs.dsn || cmdArgs.provider || cmdArgs.cns 
			 || cmdArgs.username || cmdArgs.password
			 || cmdArgs.description) && (!cmdArgs.remove))
	{
		if (!cmdArgs.dsn)
		{
			g_printerr("Error: Please specify the datasource name\n");
			out=1;
		}
		if (!cmdArgs.provider)
		{
			g_printerr("Error: Please specify the provider name\n");
			out=1;
		}
		if (!cmdArgs.cns)
		{
			g_printerr("Error: Please specify the connection string\n");
			out=1;
		}
		if (!cmdArgs.description)
		{
			g_printerr("Error: Please specify a description for the datasource\n");
			out=1;
		}
		if (!cmdArgs.username)
		{
			g_printerr("Error: Please specify the user name\n");
			out=1;
		}
		if (!cmdArgs.password)
		{
			g_printerr("Error: Please specify the password\n");
			out=1;
		}

		if(out)
			exit(-1);
	}


	poptFreeContext (context);
}

int main (int argc, char *argv[])
{
	options(argc, (const char **) argv);
	gda_init("yagda-config-tool","Version 0.1",argc,argv);
	gda_main_run ((GdaInitFunc) run_gda_config,NULL);
	return(0);
}

